#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

""" Generates a clinical-component-to-loinc.xml from CSV """
import argparse
import csv
import sys


def main():
    """ Main entry point """
    parser = argparse.ArgumentParser(
        description='Generates a clinical-component-to-loinc.xml from a CSV',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="The expected format is 4-columns without headers. \n"
               "The column order is: \n"
               "\tdescription, local-code, units, loinc-code\n"
               "\n"
               "Since the third column 'units' is not used, however, a \n"
               "3-column format is also acceptable: \n"
               "\tdescription, local-code, loinc-code",
        usage='%(prog)s < mapping.csv > clinical-component-to-loinc.xml')
    parser.parse_args()

    print '<?xml version="1.0" encoding="UTF-8"?>'
    print """
    <clinical_datum>
        <version>1.0</version>
        <Description>A mapping of local clinical component identifiers to their corresponding LOINC codes</Description>
        <components>"""

    reader = csv.reader(sys.stdin)
    for line in reader:
        length = len(line)
        if 4 == length:
            (description, code, unit, loinc) = line[0:4]
        elif 3 == length:
            (description, code, loinc) = line[0:3]
        else:
            raise Exception('Unexpected CSV format: ' + str(line))

        print """
            <component>
              <description>{description}</description>
              <source>
                <name>COMPONENT_ID</name>
                <value>{code}</value>
              </source>
              <target>
                <name>loinc_code</name>
                <value>{loinc}</value>
              </target>
            </component>""".format(description=description,
                                   code=code,
                                   loinc=loinc.rstrip())

    print "</components>"
    print "</clinical_datum>"


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

""" Converts clinical-component-to-loinc.xml to CSV """

import argparse
import sys
from xml.etree import ElementTree


def main():
    """ Main entry point """
    parser = argparse.ArgumentParser(
        description='Converts clinical-component-to-loinc.xml to CSV',
        usage='%(prog)s < clinical-component-to-loinc.xml')
    parser.parse_args()

    clinical_datum = ElementTree.fromstring(sys.stdin.read())

    for component in clinical_datum.findall('.//component'):
        print "{description}, {code}, {loinc}".format(
            description=component.findtext('description'),
            code=component.findtext('source/value'),
            loinc=component.findtext('target/value'))


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Contributors:
# Nicholas Rejack <nrejack@ufl.edu>
# Kevin Hanson <hansonks@gmail.com>
# Copyright (c) 2014-2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

# preprocessing rules go here.


# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

__copyright__ = "Copyright 2015, University of Florida"
__license__ = "BSD 3-Clause"

######################################################
#
#
#  Cancel.py module
#       This module looks for 'CANCELED', 'CANCELLED'
#       or other variations and blanks out the form
#       before sending to REDCap.
#
#
######################################################

from redi.form import Form
import re


def run_rules(data):
    """Mandatory function which runs the rules pertaining
        to this module

    """
    form = Form(data)
    for event in form.events():
        for field in event.fields():
            field_value = field.value
            if not field_value:
                continue
            else:
                is_cancelled = False
                is_cancelled = process_field_cancelled_value(field_value)
                if is_cancelled is True:
                    # clear the field value
                    field.clear_value()
                else:
                    continue
    return form

def process_field_cancelled_value(value):
    """This method processes the value in field. It checks
        for the value 'CANCELED' or 'CANCELLED' in the
        <value> tag under the parent <field> tag.
    """
    value = value.strip().lower()
    cancel_match = re.findall(r'cancel[led$]*', value, re.I)
    if not cancel_match:
        return False
    else:
        return True



# Contributors:
# Christopher P. Barnes <senrabc@gmail.com>
# Andrei Sura: github.com/indera
# Mohan Das Katragadda <mohan.das142@gmail.com>
# Philip Chase <philipbchase@gmail.com>
# Ruchi Vivek Desai <ruchivdesai@gmail.com>
# Taeber Rapczak <taeber@ufl.edu>
# Josh Hanna <josh@hanna.io>
# Copyright (c) 2015, University of Florida
# All rights reserved.
#
# Distributed under the BSD 3-Clause License
# For full text of the BSD 3-Clause License see http://opensource.org/licenses/BSD-3-Clause

__copyright__ = "Copyright 2015, University of Florida"
__license__ = "BSD 3-Clause"

import unittest
from StringIO import StringIO

from lxml import etree

import cancel
from redi.form import Form

FIELD_NAME = "test_field_name"
TEST_FIELD_VALUE1 = "CANCELLED"
TEST_FIELD_VALUE2 = "CANCELED"
TEST_FIELD_VALUE3 = "CANCEL"
TEST_FIELD_VALUE4 = "s0meJUnk"
TEST_FIELD_VALUE5 = " "
TEST_FIELD_VALUE6 = ""

class TestCancelRuleProcessing(unittest.TestCase):

    def test_cancel_present(self):
        data = ("<person_form_event><person><study_id>9001</study_id><all_form_events><form>"
                "  <name>Best. Form. Ever.</name>"
                "  <event>"
                "    <name>The Event</name>"
                "    <field>"
                "      <name>The Field</name>"
                "      <value>some value</value>"
                "    </field>"
                "    <field>"
                "      <name>{0}</name>"
                "      <value>{1}</value>"
                "    </field>"
                "  </event>"
                "</form></all_form_events></person></person_form_event>"
                .format(FIELD_NAME, TEST_FIELD_VALUE3))
        tree = parse_xml_from_string(data)

        form = cancel.run_rules(tree)

        self.check_form(form, FIELD_NAME, expected_value='')

    def test_cancelled_present(self):
        data = ("<person_form_event><person><study_id>9001</study_id><all_form_events><form>"
                "  <name>Best. Form. Ever.</name>"
                "  <event>"
                "    <name>The Event</name>"
                "    <field>"
                "      <name>The Field</name>"
                "      <value>some value</value>"
                "    </field>"
                "    <field>"
                "      <name>{0}</name>"
                "      <value>{1}</value>"
                "    </field>"
                "  </event>"
                "</form></all_form_events></person></person_form_event>"
                .format(FIELD_NAME, TEST_FIELD_VALUE1))
        tree = parse_xml_from_string(data)

        form = cancel.run_rules(tree)

        self.check_form(form, FIELD_NAME, expected_value='')

    def test_canceled_present(self):
        data = ("<person_form_event><person><study_id>9001</study_id><all_form_events><form>"
                "  <name>Best. Form. Ever.</name>"
                "  <event>"
                "    <name>The Event</name>"
                "    <field>"
                "      <name>The Field</name>"
                "      <value>some value</value>"
                "    </field>"
                "    <field>"
                "      <name>{0}</name>"
                "      <value>{1}</value>"
                "    </field>"
                "  </event>"
                "</form></all_form_events></person></person_form_event>"
                .format(FIELD_NAME, TEST_FIELD_VALUE2))
        tree = parse_xml_from_string(data)

        form = cancel.run_rules(tree)

        self.check_form(form, FIELD_NAME, expected_value='')


    def test_cancel_not_present(self):
        data = ("<person_form_event><person><study_id>9001</study_id><all_form_events><form>"
                "  <name>Best. Form. Ever.</name>"
                "  <event>"
                "    <name>The Event</name>"
                "    <field>"
                "      <name>The Field</name>"
                "      <value>some value</value>"
                "    </field>"
                "    <field>"
                "      <name>{0}</name>"
                "      <value>{1}</value>"
                "    </field>"
                "  </event>"
                "</form></all_form_events></person></person_form_event>"
                .format(FIELD_NAME, TEST_FIELD_VALUE4))
        tree = parse_xml_from_string(data)

        form = cancel.run_rules(tree)

        self.check_form(form, FIELD_NAME, expected_value=TEST_FIELD_VALUE4)

    def test_cancel_not_present_space_present(self):
        data = ("<person_form_event><person><study_id>9001</study_id><all_form_events><form>"
                "  <name>Best. Form. Ever.</name>"
                "  <event>"
                "    <name>The Event</name>"
                "    <field>"
                "      <name>The Field</name>"
                "      <value>some value</value>"
                "    </field>"
                "    <field>"
                "      <name>{0}</name>"
                "      <value>{1}</value>"
                "    </field>"
                "  </event>"
                "</form></all_form_events></person></person_form_event>"
                .format(FIELD_NAME, TEST_FIELD_VALUE5))
        tree = parse_xml_from_string(data)

        form = cancel.run_rules(tree)

        self.check_form(form, FIELD_NAME, expected_value=TEST_FIELD_VALUE5)

    def test_cancel_with_no_value(self):
        data = ("<person_form_event><person><study_id>9001</study_id><all_form_events><form>"
                "  <name>Best. Form. Ever.</name>"
                "  <event>"
                "    <name>The Event</name>"
                "    <field>"
                "      <name>The Field</name>"
                "      <value>some value</value>"
                "    </field>"
                "    <field>"
                "      <name>{0}</name>"
                "      <value>{1}</value>"
                "    </field>"
                "  </event>"
                "</form></all_form_events></person></person_form_event>"
                .format(FIELD_NAME, TEST_FIELD_VALUE6))
        tree = parse_xml_from_string(data)

        form = cancel.run_rules(tree)

        self.check_form(form, FIELD_NAME, expected_value=TEST_FIELD_VALUE6)


    def check_form(self, form, field_name, expected_value):
        for event in form.events():
            for field in event.fields():
                #print field.name+" - "+field_name+" - "+field.value
                if field.name == field_name:
                    self.assertEquals(field.value, expected_value)

def parse_xml_from_string(xml):
    return etree.parse(StringIO(xml))
